@@ -6,9 +6,12 @@ Basking.io — Python SDK
 
 import json
 import logging
+import os
 from datetime import datetime, timedelta
+from typing import Tuple, List, Iterator
 
 import pandas as pd
+import pytz
 
 
 class Occupancy:
@@ -60,62 +63,32 @@ class Occupancy:
 
         :return: DataFrame or Object with statistics about the building on the selected date range
         """
-
-        # checks
+        if (start_obj_tz_aware and end_obj_tz_unaware) or (end_obj_tz_aware and start_obj_tz_unaware):
+            raise Exception("cannot process mixed tz-aware and unaware timestamps")
+        else:
+            start_timestamp = self.basking.utils.convert_timestamp_to_building_timezone(
+                building_id,
+                start_obj_tz_aware or start_obj_tz_unaware
+            )
+            end_timestamp = self.basking.utils.convert_timestamp_to_building_timezone(
+                building_id,
+                end_obj_tz_aware or end_obj_tz_unaware
+            )
 
         try:
-            segment_start_date_timestamp = ''
-            segment_end_date_timestamp = ''
+            page_size_d = int(os.getenv('API_PAGESIZE', '30'))
+            date_slices = self._paginate_dates(start_timestamp, end_timestamp, page_size_d)
             data = []
-            page_size_d = 30
-            segment_start_date_timestamp = start_obj_tz_unaware
-            segment_end_date_timestamp = min(
-                segment_start_date_timestamp + timedelta(days=page_size_d),
-                end_obj_tz_unaware
-            )
-            while segment_start_date_timestamp < end_obj_tz_unaware:
-                try:
-                    start_unaware_timezone, end_unaware_timezone = self.aware_unaware_datetime_converter(
-                        building_id=building_id,
-                        start_obj_tz_aware=start_obj_tz_aware,
-                        end_obj_tz_aware=end_obj_tz_aware,
-                        start_obj_tz_unaware=segment_start_date_timestamp,
-                        end_obj_tz_unaware=segment_end_date_timestamp,
-                        start_unix_timestamp=start_unix_timestamp,
-                        end_unix_timestamp=end_unix_timestamp
+
+            for start_date, end_date in date_slices:
+                result = self.basking.graphql_query.graphql_executor(
+                    *self.basking.graphql_query.get_building_occupancy_stats_daily_graphql_query(
+                        building_id,
+                        start_date.strftime(self.basking.graphql_query.DATE_FORMAT),
+                        end_date.strftime(self.basking.graphql_query.DATE_FORMAT),
                     )
-
-                except TypeError:
-                    self.log.error('no data')
-                    return
-                self.log.debug(
-                    " > Paginating between timestamps: %s - %s ",
-                    segment_start_date_timestamp,
-                    segment_end_date_timestamp
                 )
-                self.log.debug('Started get_building_occupancy_stats_daily with'
-                               ' - building_id = %s'
-                               ' - start_date_str =%s'
-                               ' - end_date_str = %s',
-                               building_id, start_unaware_timezone, end_unaware_timezone)
-                query, variables = self.basking.graphql_query.get_building_occupancy_stats_daily_graphql_query(
-                    str(building_id), str(start_unaware_timezone), str(end_unaware_timezone))
-                result = self.basking.graphql_query.graphql_executor(query=query, variables=variables)
-                data_ = json.loads(result)
-                data += data_['data']['location']['occupancy']['daily']
-
-                # prepare for next loop.
-                segment_start_date_timestamp = segment_end_date_timestamp + timedelta(days=1)
-
-                segment_end_date_timestamp = min(
-                    segment_start_date_timestamp + timedelta(days=page_size_d),
-                    end_obj_tz_unaware
-                )
-
-            if not isinstance(data, object):
-                # has to be an object
-                raise AssertionError(f'wrong type for data {type(data)}. required type is object')
-            # has_errors = self.basking.basking_handle_api_return_errors(data)
+                data += json.loads(result)['data']['location']['occupancy']['daily']
 
             if pandify:
                 df = pd.DataFrame(data)
@@ -148,6 +121,15 @@ class Occupancy:
         except TypeError:
             self.log.error('no data returned for %s', building_id)
 
+    def _paginate_dates(self, start: datetime, end: datetime, page_size: int) -> Iterator[Tuple[datetime, datetime]]:
+        date_slices = []
+        slice_start = start
+        while slice_start < end:
+            next_slice_start = slice_start + timedelta(days=page_size)
+            date_slices += [(slice_start, min(next_slice_start, end) - timedelta(seconds=1))]
+            yield slice_start, min(next_slice_start, end) - timedelta(seconds=1)
+            slice_start = next_slice_start
+
     def get_building_occupancy_hourly(
             self,
             building_id,
@@ -196,84 +178,64 @@ class Occupancy:
 
         :return: results as DF or array
         """
-
-        building = self.basking.location.get_building(building_id=building_id)
-        tz_str = building['data']['getBuilding']['timeZone']
-        # pagination
-        page_size_d = 30  # 30 days max query length!
-        # initial condition
-        segment_start_date_timestamp = start_obj_tz_unaware
-
-        segment_end_date_timestamp = min(
-            segment_start_date_timestamp + timedelta(days=page_size_d),
-            end_obj_tz_unaware
-        )
-
-        data = []
-
-        while segment_start_date_timestamp < end_obj_tz_unaware:
-            try:
-                start_unaware_timezone, end_unaware_timezone = self.aware_unaware_datetime_converter(
-                    building_id=building_id,
-                    start_obj_tz_aware=start_obj_tz_aware,
-                    end_obj_tz_aware=end_obj_tz_aware,
-                    start_obj_tz_unaware=segment_start_date_timestamp,
-                    end_obj_tz_unaware=segment_end_date_timestamp,
-                    start_unix_timestamp=start_unix_timestamp,
-                    end_unix_timestamp=end_unix_timestamp
-                )
-
-            except TypeError:
-                self.log.error('no data')
-                return
-            self.log.debug(
-                " > Paginating between timestamps: %s - %s ",
-                start_unaware_timezone,
-                end_unaware_timezone
-            )
-
-            data_ = self.get_building_occupancy_hourly_pagination(
-                start_unaware_timezone,
-                end_unaware_timezone,
-                building_id,
-                ap_floor_id,
-                floor_area_id,
-                ap_id
-            )
-            if 'message' in data:
-                self.basking.api_timeout_handler(data)
-            data += data_['data']['location']['occupancy']['hourly']
-            # prepare for next loop.
-            segment_start_date_timestamp = segment_end_date_timestamp
-
-            segment_end_date_timestamp = min(
-                segment_start_date_timestamp + timedelta(days=page_size_d),
-                end_obj_tz_unaware
-            )
-
-        self.log.debug('Done with pagination')
-        if len(data):
-            if pandify:
-                df = pd.DataFrame(data)
-                df.rename(columns={
-                    'hour': 'timestamp',
-                    'occupancy': 'occupancy_hourly'
-                }, inplace=True)
-
-                df.set_index(
-                    pd.to_datetime(
-                        df['timestamp']),
-                    inplace=True)
-
-                # convert the tz to building tz
-                df.index = df.tz_convert(tz_str).index
-                df = df[['occupancy_hourly', 'adjustedCapacity', 'adjustedCapacityPct']]
-                return df
-            return data
+        if (start_obj_tz_aware and end_obj_tz_unaware) or (end_obj_tz_aware and start_obj_tz_unaware):
+            raise Exception("cannot process mixed tz-aware and unaware timestamps")
         else:
-            self.log.info('got no data')
-            if pandify:
-                return pd.DataFrame()
+            building = self.basking.location.get_building(building_id=building_id)
+            start_timestamp = self.basking.utils.convert_timestamp_to_building_timezone(
+                building_id,
+                start_obj_tz_aware or start_obj_tz_unaware
+            )
+            end_timestamp = self.basking.utils.convert_timestamp_to_building_timezone(
+                building_id,
+                end_obj_tz_aware or end_obj_tz_unaware
+            )
+
+        try:
+            page_size_d = int(os.getenv('API_PAGESIZE', '30'))
+            date_slices = self._paginate_dates(start_timestamp, end_timestamp, page_size_d)
+            data = []
+
+            for start_date, end_date in date_slices:
+                data_ = self.get_building_occupancy_hourly_pagination(
+                    start_date.isoformat() + 'Z',  # TODO: Get rid of this
+                    end_date.isoformat() + 'Z',
+                    building_id,
+                    ap_floor_id,
+                    floor_area_id,
+                    ap_id
+                )
+                if 'message' in data:
+                    self.basking.api_timeout_handler(data)
+                data += data_['data']['location']['occupancy']['hourly']
+
+            self.log.debug('Done with pagination')
+            if len(data):
+                if pandify:
+                    df = pd.DataFrame(data)
+                    df.rename(columns={
+                        'hour': 'timestamp',
+                        'occupancy': 'occupancy_hourly'
+                    }, inplace=True)
+
+                    df.set_index(
+                        pd.to_datetime(
+                            df['timestamp']),
+                        inplace=True)
+
+                    # convert the tz to building tz
+                    df.index = df.tz_convert(building['data']['getBuilding']['timeZone']).index
+                    df = df[['occupancy_hourly', 'adjustedCapacity', 'adjustedCapacityPct']]
+                    return df
+                return data
+            else:
+                self.log.info('got no data')
+                if pandify:
+                    return pd.DataFrame()
+        except Exception as e:
+            self.log.error(e)
+            raise e
+
 
     def get_building_occupancy_hourly_pagination(
             self,
@@ -802,6 +764,16 @@ class Occupancy:
         else:
             self.log.error('Building has no timezone. Aborting. ')
 
+    def convert_timestamp_to_building_timezone(self, building_id: str, timestamp: datetime) -> datetime:
+        """
+        :param building_id: the building to get the timezone from
+        :param timestamp: no timezone
+        :return: UTC representation of the timestamp in the building's timezone
+        """
+        building = self.basking.location.get_building(building_id=building_id)
+        timezoned_timestamp = pytz.timezone(building['data']['getBuilding']['timeZone']).localize(timestamp)
+        return datetime.utcfromtimestamp(timezoned_timestamp.timestamp())
+
     def aware_unaware_datetime_converter(
             self,
             building_id=None,
@@ -833,6 +805,8 @@ class Occupancy:
 
         :return: timestamp as string, timezone aware using building timezone
         """
+        print(
+            "WARNING: aware_unaware_datetime_converter is deprecated. Please use convert_timestamp_to_building_timezone")
         try:
             if isinstance(start_obj_tz_aware, datetime) & isinstance(end_obj_tz_aware, datetime):
                 if self.basking.check_if_datetime_obj_tz_aware(
